@@ -837,7 +837,7 @@ _XmHTMLFormAddSelect(XmHTMLWidget html, String attributes)
 		scrolled_win = gtk_scrolled_window_new (NULL, NULL);
 		gtk_widget_set_usize (scrolled_win, 80, 80);
 		list = gtk_list_new ();
-		gtk_container_add (GTK_CONTAINER (scrolled_win), list);
+		gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_win), list);
 		gtk_widget_show (list);
 		gtk_object_set_user_data (GTK_OBJECT (scrolled_win), list);
 		
@@ -969,7 +969,7 @@ create_file_selection (GtkEntry *entry)
 	
 	if (!window){
 		window = gtk_file_selection_new ("file selection dialog");
-		gtk_window_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
+		gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
 		
 		gtk_signal_connect (GTK_OBJECT (window), "destroy",
 				    GTK_SIGNAL_FUNC(destroy_window),
@@ -1004,7 +1004,7 @@ _XmHTMLFormAddInput(XmHTMLWidget html, String attributes)
 {
 	static XmHTMLForm *entry;
 	GtkWidget *hbox;
-	char   *chPtr;
+	gchar *chPtr = NULL;
 
 	if(attributes == NULL)
 		return(NULL);
