@@ -1,3 +1,5 @@
+#include <stdio.h>
+
 /*
  * Gtk/XmHTML widget port.
  *
@@ -268,7 +270,10 @@ gtk_xmhtml_class_init (GtkXmHTMLClass *class)
 	GtkObjectClass    *object_class;
 	GtkWidgetClass    *widget_class;
 	GtkContainerClass *container_class;
-		
+	extern FILE *__rsd_debug_file;
+
+	__rsd_debug_file = stdout;
+
 	object_class    = (GtkObjectClass *) class;
 	widget_class    = (GtkWidgetClass *) class;
 	container_class = (GtkContainerClass *) class;
@@ -726,7 +731,7 @@ gtk_xmhtml_focus (GtkWidget *widget, GdkEvent *event, gpointer closure)
 	GtkObject *htmlo = GTK_OBJECT (closure);
 	GdkEventFocus *focus = (GdkEventFocus *) event;
 	XmAnyCallbackStruct cbs;
-	int focusing_in;
+	gint focusing_in = 0;
 	
 	if (event->type == GDK_FOCUS_CHANGE)
 		focusing_in = (focus->window == html->html.work_area->window);
@@ -1472,8 +1477,9 @@ _XmHTMLCheckXCC(XmHTMLWidget html)
 		/* walk TWidget tree or get default visual */
 		if(visual == NULL){
 			/* visual = XCCGetParentVisual((TWidget)html);*/
-			fprintf (stderr, "%s: should not happen\n", __FUNCTION__);
-			exit (1);
+		  fprintf(stderr,
+			  "XmHTML.c: _XmHTMLCheckXCC:: should not happen\n");
+		  exit (1);
 		}
 		/* create an xcc for this TWidget */
 		html->html.xcc = XCCCreate((TWidget)html, visual, cmap);
@@ -2251,3 +2257,50 @@ gtk_signal_handler_pending (GtkObject *object, gint signal_type, gboolean may_be
 	return 1;
 }
 #endif
+
+void
+Toolkit_Draw_Shadows(XmHTMLWidget html, GC* top_shadow, GC* bottom_shadow,
+		     gint xs, gint ys, gint w, gint h, gint shadow_thick, gint shadow_type)
+{
+	switch (shadow_type)
+	  {
+	  case XmSHADOW_IN:
+	    gtk_draw_shadow(GTK_WIDGET(html)->style, GTK_WIDGET(html)->window,
+			    GTK_STATE_NORMAL, GTK_SHADOW_IN,
+			    xs, ys,
+			    w, h);
+	    break;
+	  case XmSHADOW_OUT:
+	    gtk_draw_shadow(GTK_WIDGET(html)->style, GTK_WIDGET(html)->window,
+			    GTK_STATE_NORMAL, GTK_SHADOW_OUT,
+			    xs, ys,
+			    w, h);
+	    break;
+	  default:
+	    break;
+	    ;
+	  }
+}
+
+void
+_XmHTMLRecomputeShadowColors(XmHTMLWidget html, Pixel base) 
+{
+	/* 
+	* We can only compute the colors when we have a GC. If we don't
+	* have a GC, the widget is not yet realized. Use managers defaults
+	* then.
+	*/
+#if 0
+	if(html->html.gc != NULL)
+	{
+		Pixel top = None, bottom = None;
+		Arg args[2];
+
+		XmGetColors(XtScreen((Widget)html), html->core.colormap,
+			base, NULL, &top, &bottom, NULL);
+		XtSetArg(args[0], XmNtopShadowColor, top);
+		XtSetArg(args[1], XmNbottomShadowColor, bottom);
+		XtSetValues((Widget)html, args, 2);
+	}
+#endif
+}
