@@ -13,8 +13,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
  */
 
 #include <config.h>
@@ -55,6 +55,13 @@ gnome_app_interactive_statusbar(GnomeApp * app)
 	  gnome_preferences_get_statusbar_interactive() );
 }
 
+static void gnome_app_activate_statusbar(GnomeApp *app)
+{
+  gtk_window_set_focus(GTK_WINDOW(app), GNOME_APPBAR(app->statusbar)->status);
+  gtk_window_activate_focus(GTK_WINDOW(app));
+  gdk_window_raise(GTK_WIDGET(app)->window);
+}
+
 /* ================================================================== */
 
 /* =================================================================== */
@@ -85,6 +92,7 @@ static void gnome_app_message_bar (GnomeApp * app, const gchar * message)
 {
   gchar * prompt = g_copy_strings(message, _(" (press return)"), NULL);
   gnome_appbar_set_prompt(GNOME_APPBAR(app->statusbar), prompt, FALSE);
+  gnome_app_activate_statusbar(app);
   g_free(prompt);
   gtk_signal_connect(GTK_OBJECT(app->statusbar), "user_response",
 		     GTK_SIGNAL_FUNC(ack_cb), NULL);
@@ -92,6 +100,20 @@ static void gnome_app_message_bar (GnomeApp * app, const gchar * message)
 		     GTK_SIGNAL_FUNC(ack_clear_prompt_cb), NULL);
 }
 
+
+/**
+ * gnome_app_message
+ * @app: Pointer to GNOME app object
+ * @message: Text of message to be displayed
+ *
+ * Description: A simple message, in an OK dialog or the status bar.
+ * Requires confirmation from the user before it goes away.
+ *
+ * Returns:
+ * Pointer to dialog widget, or %NULL if error or message in
+ * status bar.
+ **/
+
 GtkWidget *
 gnome_app_message (GnomeApp * app, const gchar * message)
 {
@@ -117,6 +139,21 @@ gnome_app_error_bar(GnomeApp * app, const gchar * error)
   g_free(s);
 }
 
+
+/**
+ * gnome_app_error
+ * @app: Pointer to GNOME app object
+ * @error: Text of error message to be displayed
+ *
+ * Description:
+ * An important fatal error; if it appears in the statusbar, 
+ * it might gdk_beep() and require acknowledgement.
+ *
+ * Returns:
+ * Pointer to dialog widget, or %NULL if error or message in
+ * status bar.
+ **/
+
 GtkWidget * 
 gnome_app_error (GnomeApp * app, const gchar * error)
 {
@@ -139,6 +176,20 @@ static void gnome_app_warning_bar (GnomeApp * app, const gchar * warning)
   g_free(s);
 }
 
+
+/**
+ * gnome_app_warning
+ * @app: Pointer to GNOME app object
+ * @warning: Text of warning message to be displayed
+ *
+ * Description:
+ * A not-so-important error, but still marked better than a flash
+ *
+ * Returns:
+ * Pointer to dialog widget, or %NULL if error or message in
+ * status bar.
+ **/
+
 GtkWidget * 
 gnome_app_warning (GnomeApp * app, const gchar * warning)
 {
@@ -186,6 +237,18 @@ remove_timeout_cb ( GtkWidget * app, MessageInfo * mi )
 
 static const guint32 flash_length = 3000; /* 3 seconds, I hope */
 
+
+/**
+ * gnome_app_flash
+ * @app: Pointer to GNOME app object
+ * @flash: Text of message to be flashed
+ *
+ * Description:
+ * Flash the message in the statusbar for a few moments; if no
+ * statusbar, do nothing (?). For trivial little status messages,
+ * e.g. "Auto saving..."
+ **/
+
 void 
 gnome_app_flash (GnomeApp * app, const gchar * flash)
 {
@@ -286,6 +349,7 @@ gnome_app_reply_bar(GnomeApp * app, const gchar * question,
   prompt = g_copy_strings(question, yes_or_ok ? _(" (yes or no)") : 
 			  _("  - OK? (yes or no)"), NULL);
   gnome_appbar_set_prompt(GNOME_APPBAR(app->statusbar), prompt, modal);
+  gnome_app_activate_statusbar(app);
   g_free(prompt);
   
   ri = g_new(ReplyInfo, 1);
@@ -298,6 +362,22 @@ gnome_app_reply_bar(GnomeApp * app, const gchar * question,
 		     GTK_SIGNAL_FUNC(reply_clear_prompt_cb), ri);
 }
 
+
+/**
+ * gnome_app_question
+ * @app: Pointer to GNOME app object
+ * @question: Text of question to be displayed
+ * @callback:
+ * @data:
+ *
+ * Description:
+ * Ask a yes or no question, and call the callback when it's answered.
+ *
+ * Returns:
+ * Pointer to dialog widget, or %NULL if error or message in
+ * status bar.
+ **/
+
 GtkWidget * 
 gnome_app_question (GnomeApp * app, const gchar * question,
 		    GnomeReplyCallback callback, gpointer data)
@@ -317,6 +397,22 @@ gnome_app_question (GnomeApp * app, const gchar * question,
   }
 }
 
+
+/**
+ * gnome_app_question_modal
+ * @app: Pointer to GNOME app object
+ * @question: Text of question to be displayed
+ * @callback:
+ * @data:
+ *
+ * Description:
+ * Ask a yes or no question, and call the callback when it's answered.
+ *
+ * Returns:
+ * Pointer to dialog widget, or %NULL if error or message in
+ * status bar.
+ **/
+
 GtkWidget * 
 gnome_app_question_modal (GnomeApp * app, const gchar * question,
 			  GnomeReplyCallback callback, gpointer data)
@@ -336,6 +432,21 @@ gnome_app_question_modal (GnomeApp * app, const gchar * question,
   }
 }
 
+
+/**
+ * gnome_app_ok_cancel
+ * @app: Pointer to GNOME app object
+ * @message: Text of message to be displayed
+ * @callback:
+ * @data:
+ *
+ * Description:
+ *
+ * Returns:
+ * Pointer to dialog widget, or %NULL if error or message in
+ * status bar.
+ **/
+
 GtkWidget * 
 gnome_app_ok_cancel (GnomeApp * app, const gchar * message,
 		     GnomeReplyCallback callback, gpointer data)
@@ -355,6 +466,21 @@ gnome_app_ok_cancel (GnomeApp * app, const gchar * message,
   }
 }
 
+
+/**
+ * gnome_app_ok_cancel_modal
+ * @app: Pointer to GNOME app object
+ * @message: Text of message to be displayed
+ * @callback:
+ * @data:
+ *
+ * Description:
+ *
+ * Returns:
+ * Pointer to dialog widget, or %NULL if error or message in
+ * status bar.
+ **/
+
 GtkWidget * 
 gnome_app_ok_cancel_modal (GnomeApp * app, const gchar * message,
 			   GnomeReplyCallback callback, gpointer data)
@@ -441,6 +567,21 @@ gnome_app_request_bar  (GnomeApp * app, const gchar * prompt,
   }
 }
 
+
+/**
+ * gnome_app_request_string
+ * @app: Pointer to GNOME app object
+ * @prompt: Text of prompt to be displayed
+ * @callback:
+ * @data:
+ *
+ * Description:
+ *
+ * Returns:
+ * Pointer to dialog widget, or %NULL if error or message in
+ * status bar.
+ **/
+
 GtkWidget * 
 gnome_app_request_string (GnomeApp * app, const gchar * prompt,
 			  GnomeStringCallback callback, gpointer data)
@@ -461,6 +602,21 @@ gnome_app_request_string (GnomeApp * app, const gchar * prompt,
 }
 
 
+
+/**
+ * gnome_app_request_password
+ * @app: Pointer to GNOME app object
+ * @prompt: Text of prompt to be displayed
+ * @callback:
+ * @data:
+ *
+ * Description:
+ *
+ * Returns:
+ * Pointer to dialog widget, or %NULL if error or message in
+ * status bar.
+ **/
+
 GtkWidget * 
 gnome_app_request_password (GnomeApp * app, const gchar * prompt,
 			    GnomeStringCallback callback, gpointer data)
@@ -557,6 +713,20 @@ stop_progress_cb(GnomeApp * app, GnomeAppProgressKey key)
 
 /* FIXME share code between manual and timeout */
 
+/**
+ * gnome_app_progress_timeout
+ * @app:
+ * @description:
+ * @interval:
+ * @percentage_cb:
+ * @cancel_cb:
+ * @data:
+ *
+ * Description:
+ *
+ * Returns:
+ **/
+
 GnomeAppProgressKey 
 gnome_app_progress_timeout (GnomeApp * app, 
 			    const gchar * description,
@@ -599,6 +769,19 @@ gnome_app_progress_timeout (GnomeApp * app,
   return key;
 }
 
+
+/**
+ * gnome_app_progress_manual
+ * @app:
+ * @description:
+ * @cancel_cb:
+ * @data:
+ *
+ * Description:
+ *
+ * Returns:
+ **/
+
 GnomeAppProgressKey 
 gnome_app_progress_manual (GnomeApp * app, 
 			   const gchar * description,
@@ -634,6 +817,14 @@ gnome_app_progress_manual (GnomeApp * app,
   return key;
 }
 
+/**
+ * gnome_app_set_progress
+ * @key:
+ * @percent:
+ *
+ * Description:
+ **/
+
 void gnome_app_set_progress (GnomeAppProgressKey key, gdouble percent)
 {
   ProgressKeyReal * real_key = (GnomeAppProgressKey) key;
@@ -656,6 +847,13 @@ static void progress_timeout_remove(ProgressKeyReal * key)
   }
 }
 
+/**
+ * gnome_app_progress_done
+ * @key:
+ *
+ * Description:
+ **/
+
 void gnome_app_progress_done (GnomeAppProgressKey key)
 {
   ProgressKeyReal * real_key = (GnomeAppProgressKey) key;
