@@ -11,6 +11,7 @@
 #include <config.h>
 #include <math.h>
 #include "gnome-canvas-image.h"
+#include "gnometypebuiltins.h"
 
 
 enum {
@@ -34,7 +35,7 @@ static void gnome_canvas_image_get_arg    (GtkObject             *object,
 					   GtkArg                *arg,
 					   guint                  arg_id);
 
-static void   gnome_canvas_image_reconfigure (GnomeCanvasItem *item);
+static void   gnome_canvas_image_update      (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags);
 static void   gnome_canvas_image_realize     (GnomeCanvasItem *item);
 static void   gnome_canvas_image_unrealize   (GnomeCanvasItem *item);
 static void   gnome_canvas_image_draw        (GnomeCanvasItem *item, GdkDrawable *drawable,
@@ -42,6 +43,7 @@ static void   gnome_canvas_image_draw        (GnomeCanvasItem *item, GdkDrawable
 static double gnome_canvas_image_point       (GnomeCanvasItem *item, double x, double y,
 					      int cx, int cy, GnomeCanvasItem **actual_item);
 static void   gnome_canvas_image_translate   (GnomeCanvasItem *item, double dx, double dy);
+static void   gnome_canvas_image_bounds      (GnomeCanvasItem *item, double *x1, double *y1, double *x2, double *y2);
 
 
 static GnomeCanvasItemClass *parent_class;
@@ -81,7 +83,7 @@ gnome_canvas_image_class_init (GnomeCanvasImageClass *class)
 
 	parent_class = gtk_type_class (gnome_canvas_item_get_type ());
 
-	gtk_object_add_arg_type ("GnomeCanvasImage::image", GTK_TYPE_POINTER, GTK_ARG_READWRITE, ARG_IMAGE);
+	gtk_object_add_arg_type ("GnomeCanvasImage::image", GTK_TYPE_GDK_IMLIB_IMAGE, GTK_ARG_READWRITE, ARG_IMAGE);
 	gtk_object_add_arg_type ("GnomeCanvasImage::x", GTK_TYPE_DOUBLE, GTK_ARG_READWRITE, ARG_X);
 	gtk_object_add_arg_type ("GnomeCanvasImage::y", GTK_TYPE_DOUBLE, GTK_ARG_READWRITE, ARG_Y);
 	gtk_object_add_arg_type ("GnomeCanvasImage::width", GTK_TYPE_DOUBLE, GTK_ARG_READWRITE, ARG_WIDTH);
@@ -92,12 +94,13 @@ gnome_canvas_image_class_init (GnomeCanvasImageClass *class)
 	object_class->set_arg = gnome_canvas_image_set_arg;
 	object_class->get_arg = gnome_canvas_image_get_arg;
 
-	item_class->reconfigure = gnome_canvas_image_reconfigure;
+	item_class->update = gnome_canvas_image_update;
 	item_class->realize = gnome_canvas_image_realize;
 	item_class->unrealize = gnome_canvas_image_unrealize;
 	item_class->draw = gnome_canvas_image_draw;
 	item_class->point = gnome_canvas_image_point;
 	item_class->translate = gnome_canvas_image_translate;
+	item_class->bounds = gnome_canvas_image_bounds;
 }
 
 static void
@@ -128,10 +131,14 @@ free_pixmap_and_mask (GnomeCanvasImage *image)
 static void
 gnome_canvas_image_destroy (GtkObject *object)
 {
+	GnomeCanvasImage *image;
+
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GNOME_IS_CANVAS_IMAGE (object));
 
-	free_pixmap_and_mask (GNOME_CANVAS_IMAGE (object));
+	image = GNOME_CANVAS_IMAGE (object);
+
+	free_pixmap_and_mask (image);
 
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
@@ -256,7 +263,7 @@ gnome_canvas_image_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	}
 
 	if (update)
-		(* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->reconfigure) (item);
+		(* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->update) (item, NULL, NULL, 0);
 
 	if (calc_bounds)
 		recalc_bounds (image);
@@ -301,14 +308,14 @@ gnome_canvas_image_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 }
 
 static void
-gnome_canvas_image_reconfigure (GnomeCanvasItem *item)
+gnome_canvas_image_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags)
 {
 	GnomeCanvasImage *image;
 
 	image = GNOME_CANVAS_IMAGE (item);
 
-	if (parent_class->reconfigure)
-		(* parent_class->reconfigure) (item);
+	if (parent_class->update)
+		(* parent_class->update) (item, affine, clip_path, flags);
 
 	free_pixmap_and_mask (image);
 
@@ -333,7 +340,7 @@ gnome_canvas_image_realize (GnomeCanvasItem *item)
 		(* parent_class->realize) (item);
 
 	image->gc = gdk_gc_new (item->canvas->layout.bin_window);
-	(* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->reconfigure) (item);
+	(* GNOME_CANVAS_ITEM_CLASS (item->object.klass)->update) (item, NULL, NULL, 0);
 }
 
 static void
@@ -510,3 +517,55 @@ gnome_canvas_image_translate (GnomeCanvasItem *item, double dx, double dy)
 
 	recalc_bounds (image);
 }
+
+static void
+gnome_canvas_image_bounds (GnomeCanvasItem *item, double *x1, double *y1, double *x2, double *y2)
+{
+	GnomeCanvasImage *image;
+
+	image = GNOME_CANVAS_IMAGE (item);
+
+	*x1 = image->x;
+	*y1 = image->y;
+
+	switch (image->anchor) {
+	case GTK_ANCHOR_NW:
+	case GTK_ANCHOR_W:
+	case GTK_ANCHOR_SW:
+		break;
+
+	case GTK_ANCHOR_N:
+	case GTK_ANCHOR_CENTER:
+	case GTK_ANCHOR_S:
+		*x1 -= image->width / 2.0;
+		break;
+
+	case GTK_ANCHOR_NE:
+	case GTK_ANCHOR_E:
+	case GTK_ANCHOR_SE:
+		*x1 -= image->width;
+		break;
+	}
+
+	switch (image->anchor) {
+	case GTK_ANCHOR_NW:
+	case GTK_ANCHOR_N:
+	case GTK_ANCHOR_NE:
+		break;
+
+	case GTK_ANCHOR_W:
+	case GTK_ANCHOR_CENTER:
+	case GTK_ANCHOR_E:
+		*y1 -= image->height / 2.0;
+		break;
+
+	case GTK_ANCHOR_SW:
+	case GTK_ANCHOR_S:
+	case GTK_ANCHOR_SE:
+		*y1 -= image->height;
+		break;
+	}
+
+	*x2 = *x1 + image->width;
+	*y2 = *y1 + image->height;
+}
