@@ -156,8 +156,9 @@ gnome_entry_destroy (GtkObject *object)
 	entry = gnome_entry_gtk_entry (gentry);
 	text = gtk_entry_get_text (GTK_ENTRY (entry));
 
-	if (strcmp (text, "") != 0)
+	if (gentry->changed && (strcmp (text, "") != 0)) {
 		gnome_entry_prepend_history (gentry, TRUE, text);
+	}
 
 	gnome_entry_save_history (gentry);
 
@@ -232,6 +233,10 @@ gnome_entry_prepend_history (GnomeEntry *gentry, int save, char *text)
 
 	gtk_entry_set_text (GTK_ENTRY (entry), tmp);
 	g_free (tmp);
+
+	/* gtk_entry_set_text runs our 'entry_changed' routine, so we have
+	   to undo the effect */
+	gentry->changed = FALSE;
 }
 
 void
@@ -264,6 +269,9 @@ gnome_entry_append_history (GnomeEntry *gentry, int save, char *text)
 
 	gtk_entry_set_text (GTK_ENTRY (entry), tmp);
 	g_free (tmp);
+	/* gtk_entry_set_text runs our 'entry_changed' routine, so we have
+	   to undo the effect */
+	gentry->changed = FALSE;
 }
 
 static void
@@ -279,8 +287,6 @@ set_combo_items (GnomeEntry *gentry)
 
 	gtklist = GTK_LIST (GTK_COMBO (gentry)->list);
 
-	gtk_container_block_resize (GTK_CONTAINER (gtklist));
-
 	/* We save the contents of the entry because when we insert
 	 * items on the combo list, the contents of the entry will get
 	 * changed.
@@ -307,7 +313,7 @@ set_combo_items (GnomeEntry *gentry)
 	gtk_entry_set_text (GTK_ENTRY (entry), tmp);
 	g_free (tmp);
 
-	gtk_container_unblock_resize (GTK_CONTAINER (gtklist));
+	gentry->changed = FALSE;
 }
 
 void
@@ -316,7 +322,7 @@ gnome_entry_load_history (GnomeEntry *gentry)
 	char *prefix;
 	struct item *item;
 	int n;
-	char key[32];
+	char key[13];
 	char *value;
 
 	g_return_if_fail (gentry != NULL);
@@ -341,7 +347,7 @@ gnome_entry_load_history (GnomeEntry *gentry)
 		item->save = TRUE;
 		item->text = value;
 
-		gentry->items = g_list_prepend (gentry->items, item);
+		gentry->items = g_list_append (gentry->items, item);
 	}
 
 	set_combo_items (gentry);
@@ -356,7 +362,7 @@ gnome_entry_save_history (GnomeEntry *gentry)
 	GList *items;
 	struct item *item;
 	int n;
-	char key[32];
+	char key[13];
 
 	g_return_if_fail (gentry != NULL);
 	g_return_if_fail (GNOME_IS_ENTRY (gentry));
