@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
  */
 
 #include <config.h>
@@ -48,14 +48,14 @@
 
 struct _FontInfo
 {
-  char *family;         /* The font family this info struct describes. */
-  int *foundries;       /* An array of valid foundries. */
-  int *weights;         /* An array of valid weights. */
-  int *slants;          /* An array of valid slants. */
-  int *set_widths;      /* An array of valid set widths. */
-  int *spacings;        /* An array of valid spacings */
-  int **combos;         /* An array of valid combinations of the above 5 items */
-  int ncombos;          /* The number of elements in the "combos" array */
+  gchar *family;         /* The font family this info struct describes. */
+  gint *foundries;       /* An array of valid foundries. */
+  gint *weights;         /* An array of valid weights. */
+  gint *slants;          /* An array of valid slants. */
+  gint *set_widths;      /* An array of valid set widths. */
+  gint *spacings;        /* An array of valid spacings */
+  gint **combos;         /* An array of valid combinations of the above 5 items */
+  gint ncombos;          /* The number of elements in the "combos" array */
   GSList *fontnames;   /* An list of valid fontnames.
 			 * This is used to make sure a family/foundry/weight/slant/set_width
 			 *  combination is valid.
@@ -80,23 +80,26 @@ static void       text_set_width_callback (GtkWidget *, gpointer);
 static void       text_spacing_callback   (GtkWidget *, gpointer);
 static gint       text_size_key_function  (GtkWidget *, GdkEventKey *, gpointer);
 static void       text_font_item_update   (GtkWidget *, gpointer);
-static void       text_validate_combo     (GnomeFontSelector *, int);
+static void       text_validate_combo     (GnomeFontSelector *, gint);
 
 static void       text_get_fonts          (GnomeFontSelectorClass *klass);
-static void       text_insert_font        (FontInfo **, int *, char *);
-static GSList*    text_insert_field       (GSList *, char *, int);
-static char*      text_get_field          (char *, int);
-static int        text_field_to_index     (char **, int, char *);
-static int        text_is_xlfd_font_name  (char *);
+static void       text_insert_font        (FontInfo **, gint *, const gchar *);
+static GSList*    text_insert_field       (GSList *, const gchar *, gint);
+static gchar*     text_get_field          (const gchar *, gint);
+static gint       text_field_to_index     (gchar **,
+					   gint, const gchar *);
+static gint       text_is_xlfd_font_name  (const gchar *);
 
-static gchar *    text_get_xlfd           (double, int, char *, char *, char *,
-					   char *, char *, char *);
-static int        text_load_font          (GnomeFontSelector *);
+static gchar *    text_get_xlfd           (gdouble, gint, const gchar *,
+					   const gchar *, const gchar *,
+					   const gchar *, const gchar *,
+					   const gchar *);
+static gint       text_load_font          (GnomeFontSelector *);
 
 typedef GtkSignalFunc MenuItemCallback;
 typedef struct {
-  char *label;
-  int foo, bar;
+  gchar *label;
+  gint foo, bar;
   gpointer cb, cbdata, blah, baz;
   
 } MenuItem;
@@ -172,7 +175,7 @@ gnome_font_selector_init(GtkWidget *widget)
   /* Create the shell and vertical & horizontal boxes */
   gtk_window_set_title (GTK_WINDOW (text_tool), "Font Selector");
   gtk_window_set_policy (GTK_WINDOW (text_tool), FALSE, TRUE, TRUE);
-  gtk_window_position (GTK_WINDOW (text_tool), GTK_WIN_POS_MOUSE);
+  gtk_window_set_position (GTK_WINDOW (text_tool), GTK_WIN_POS_MOUSE);
 
   /* handle the wm close signal */
   gtk_signal_connect (GTK_OBJECT (text_tool), "delete_event",
@@ -180,7 +183,7 @@ gnome_font_selector_init(GtkWidget *widget)
 		      text_tool);
 
   text_tool->main_vbox = gtk_vbox_new (FALSE, 2);
-  gtk_container_border_width (GTK_CONTAINER (text_tool->main_vbox), 2);
+  gtk_container_set_border_width (GTK_CONTAINER (text_tool->main_vbox), 2);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (text_tool)->vbox), text_tool->main_vbox, TRUE, TRUE, 0);
   top_hbox = gtk_hbox_new (FALSE, 1);
   gtk_box_pack_start (GTK_BOX (text_tool->main_vbox), top_hbox, TRUE, TRUE, 0);
@@ -190,7 +193,7 @@ gnome_font_selector_init(GtkWidget *widget)
   gtk_widget_set_usize (list_box, FONT_LIST_WIDTH, FONT_LIST_HEIGHT);
   gtk_box_pack_start (GTK_BOX (top_hbox), list_box, TRUE, TRUE, 0);
   text_tool->font_list = gtk_list_new ();
-  gtk_container_add (GTK_CONTAINER (list_box), text_tool->font_list);
+  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (list_box), text_tool->font_list);
   gtk_list_set_selection_mode (GTK_LIST (text_tool->font_list), GTK_SELECTION_BROWSE);
 
   for (i = 0; i < nfonts; i++)
@@ -639,8 +642,8 @@ text_size_key_function (GtkWidget   *w,
 			gpointer     data)
 {
   GnomeFontSelector *text_tool;
-  char buffer[16];
-  int old_value;
+  gchar buffer[16];
+  gint old_value;
 
   text_tool = (GnomeFontSelector *) data;
 
@@ -651,7 +654,7 @@ text_size_key_function (GtkWidget   *w,
       old_value = atoi (gtk_entry_get_text (GTK_ENTRY (text_tool->size_text)));
       if (!text_load_font (text_tool))
 	{
-	  sprintf (buffer, "%d", old_value);
+	  g_snprintf (buffer, sizeof(buffer), "%d", old_value);
 	  gtk_entry_set_text (GTK_ENTRY (text_tool->size_text), buffer);
 	}
       return TRUE;
@@ -662,7 +665,7 @@ text_size_key_function (GtkWidget   *w,
 
 static void
 text_validate_combo (GnomeFontSelector *text_tool,
-		     int       which)
+		     gint       which)
 {
   FontInfo *font;
   int which_val;
@@ -805,13 +808,13 @@ text_validate_combo (GnomeFontSelector *text_tool,
 static void
 text_get_fonts (GnomeFontSelectorClass *klass)
 {
-  char **fontnames;
-  char *fontname;
-  char *field;
+  gchar **fontnames;
+  gchar *fontname;
+  gchar *field;
   GSList * temp_list;
-  int num_fonts;
-  int index;
-  int i, j;
+  gint num_fonts;
+  gint index;
+  gint i, j;
 
   /* construct a valid font pattern */
 
@@ -966,14 +969,14 @@ text_get_fonts (GnomeFontSelectorClass *klass)
 }
 
 static void
-text_insert_font (FontInfo **table,
-		  int       *ntable,
-		  char      *fontname)
+text_insert_font (FontInfo    **table,
+		  gint        *ntable,
+		  const gchar *fontname)
 {
   FontInfo *temp_info;
-  char *family;
-  int lower, upper;
-  int middle, cmp;
+  gchar *family;
+  gint lower, upper;
+  gint middle, cmp;
 
   /* insert a fontname into a table */
   family = text_get_field (fontname, FAMILY);
@@ -1037,15 +1040,15 @@ text_insert_font (FontInfo **table,
 }
 
 static GSList *
-text_insert_field (GSList *  list,
-		   char     *fontname,
-		   int       field_num)
+text_insert_field (GSList      *list,
+		   const gchar *fontname,
+		   gint        field_num)
 {
   GSList *temp_list;
   GSList *prev_list;
   GSList *new_list;
-  char *field;
-  int cmp;
+  gchar *field;
+  gint cmp;
 
   field = text_get_field (fontname, field_num);
   if (!field)
@@ -1094,12 +1097,12 @@ text_insert_field (GSList *  list,
     return new_list;
 }
 
-static char*
-text_get_field (char *fontname,
-		int   field_num)
+static gchar*
+text_get_field (const gchar *fontname,
+		gint   field_num)
 {
-  char *t1, *t2;
-  char *field;
+  const gchar *t1, *t2;
+  gchar *field;
 
   /* we assume this is a valid fontname...that is, it has 14 fields */
 
@@ -1124,11 +1127,11 @@ text_get_field (char *fontname,
 }
 
 static int
-text_field_to_index (char **table,
-		     int    ntable,
-		     char  *field)
+text_field_to_index (gchar **table,
+		     gint  ntable,
+		     const gchar  *field)
 {
-  int i;
+  gint i;
 
   for (i = 0; i < ntable; i++)
     if (strcmp (field, table[i]) == 0)
@@ -1137,10 +1140,10 @@ text_field_to_index (char **table,
   return -1;
 }
 
-static int
-text_is_xlfd_font_name (char *fontname)
+static gint
+text_is_xlfd_font_name (const gchar *fontname)
 {
-  int i;
+  gint i;
 
   i = 0;
   while (*fontname)
@@ -1151,28 +1154,28 @@ text_is_xlfd_font_name (char *fontname)
 }
 
 static gchar *
-text_get_xlfd (double  size,
-	       int     size_type,
-	       char   *foundry,
-	       char   *family,
-	       char   *weight,
-	       char   *slant,
-	       char   *set_width,
-	       char   *spacing)
+text_get_xlfd (gdouble     size,
+	       gint        size_type,
+	       const gchar *foundry,
+	       const gchar *family,
+	       const gchar *weight,
+	       const gchar *slant,
+	       const gchar *set_width,
+	       const gchar *spacing)
 {
-  char pixel_size[12], point_size[12];
+  gchar pixel_size[12], point_size[12];
 
   if (size > 0)
     {
       switch (size_type)
 	{
 	case PIXELS:
-	  sprintf (pixel_size, "%d", (int) size);
-	  sprintf (point_size, "*");
+	  g_snprintf (pixel_size, sizeof(pixel_size), "%d", (int) size);
+	  g_snprintf (point_size, sizeof(point_size), "*");
 	  break;
 	case POINTS:
-	  sprintf (pixel_size, "*");
-	  sprintf (point_size, "%d", (int) (size * 10));
+	  g_snprintf (pixel_size, sizeof(pixel_size), "*");
+	  g_snprintf (point_size, sizeof(point_size), "%d", (int) (size * 10));
 	  break;
 	}
 
@@ -1190,19 +1193,19 @@ text_get_xlfd (double  size,
     return NULL;
 }
 
-static int
+static gint
 text_load_font (GnomeFontSelector *text_tool)
 {
   GdkFont *font;
   gchar *fontname;
-  double size;
-  char *size_text;
-  char *foundry_str;
-  char *family_str;
-  char *weight_str;
-  char *slant_str;
-  char *set_width_str;
-  char *spacing_str;
+  gdouble size;
+  gchar *size_text;
+  gchar *foundry_str;
+  gchar *family_str;
+  gchar *weight_str;
+  gchar *slant_str;
+  gchar *set_width_str;
+  gchar *spacing_str;
   GnomeFontSelectorClass *klass;
 
   klass = GNOME_FONT_SELECTOR_CLASS(GTK_OBJECT(text_tool)->klass);
@@ -1249,14 +1252,14 @@ text_load_font (GnomeFontSelector *text_tool)
 gchar *
 gnome_font_selector_get_selected (GnomeFontSelector *text_tool)
 {
-  double size;
-  char *size_text;
-  char *foundry_str;
-  char *family_str;
-  char *weight_str;
-  char *slant_str;
-  char *set_width_str;
-  char *spacing_str;
+  gdouble size;
+  gchar *size_text;
+  gchar *foundry_str;
+  gchar *family_str;
+  gchar *weight_str;
+  gchar *slant_str;
+  gchar *set_width_str;
+  gchar *spacing_str;
   GnomeFontSelectorClass *klass;
 
   klass = GNOME_FONT_SELECTOR_CLASS(GTK_OBJECT(text_tool)->klass);
@@ -1461,7 +1464,7 @@ gchar *gnome_font_select_with_default(const gchar *default_font)
 	
 	text_tool->size_type = old_value;
 	
-	sprintf (buffer, "%d", old_size);
+	g_snprintf (buffer, sizeof(buffer), "%d", old_size);
 	gtk_entry_set_text (GTK_ENTRY (text_tool->size_text), buffer);
       }
     }
