@@ -517,12 +517,22 @@ sync_selection (Gil *gil, int pos, SyncType type)
 static int
 icon_event (Gil *gil, Icon *icon, GdkEvent *event)
 {
+	GList *l;
+	int n;
+	
 	switch (event->type){
 	case GDK_BUTTON_PRESS:
 		gil->last_clicked = icon;
-		if (icon->text->selected && (event->button.button == 1))
+		
+		if (icon->text->selected && (event->button.button == 1 || event->button.button == 3)){
 			gil->last_clicked = icon;
-		else {
+		
+			for (n = 0, l = gil->icon_list; l; l = l->next, n++)
+				if (l->data == icon)
+					break;
+			
+			gtk_signal_emit (GTK_OBJECT (gil), gil_signals [SELECT_ICON], n, event);
+		} else {
 			gil->last_clicked = NULL;
 			toggle_icon (gil, icon, event);
 		}
@@ -549,7 +559,7 @@ static int
 image_event (GnomeCanvasImage *img, GdkEvent *event, Icon *icon)
 {
 	Gil *gil = GIL (GNOME_CANVAS_ITEM (icon->text)->canvas);
-				      
+
 	if (icon_event (gil, icon, event))
 		return 1;
 	
@@ -1708,6 +1718,13 @@ gnome_icon_list_freeze (GnomeIconList *gil)
 	g_return_if_fail (IS_GIL (gil));
 
 	gil->frozen++;
+
+	/* We hide the root so that the user will not see any changes while the
+	 * icon list is doing stuff.
+	 */
+
+	if (gil->frozen == 1)
+		gnome_canvas_item_hide (GNOME_CANVAS (gil)->root);
 }
 
 /**
@@ -1732,6 +1749,9 @@ gnome_icon_list_thaw (GnomeIconList *gil)
 
 	gil_layout_all_icons (gil);
 	gil_scrollbar_adjust (gil);
+
+	if (gil->frozen == 0)
+		gnome_canvas_item_show (GNOME_CANVAS (gil)->root);
 }
 
 /**
