@@ -130,6 +130,7 @@ BEGIN_GNOME_DECLS
 #define GNOME_STOCK_PIXMAP_SPELLCHECK  "Spellchecker"
 #define GNOME_STOCK_PIXMAP_MIC         "Microphone"
 #define GNOME_STOCK_PIXMAP_LINE_IN     "Line In"
+#define GNOME_STOCK_PIXMAP_CDROM       "Cdrom"
 #define GNOME_STOCK_PIXMAP_VOLUME      "Volume"
 #define GNOME_STOCK_PIXMAP_BOOK_RED    "Book Red"
 #define GNOME_STOCK_PIXMAP_BOOK_GREEN  "Book Green"
@@ -474,6 +475,7 @@ GtkWidget             *gnome_stock_or_ordinary_button (const char *type);
 #define GNOME_STOCK_MENU_SPELLCHECK   "Menu_Spellchecker"
 #define GNOME_STOCK_MENU_MIC          "Menu_Microphone"
 #define GNOME_STOCK_MENU_LINE_IN      "Menu_Line In"
+#define GNOME_STOCK_MENU_CDROM	      "Menu_Cdrom"
 #define GNOME_STOCK_MENU_VOLUME       "Menu_Volume"
 #define GNOME_STOCK_MENU_BOOK_RED     "Menu_Book Red"
 #define GNOME_STOCK_MENU_BOOK_GREEN   "Menu_Book Green"
