@@ -12,8 +12,8 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
  */
 #include <math.h>
 #include <stdio.h>
@@ -721,8 +721,8 @@ gtk_dial_paint (GtkDial *dial)
 
      t_x = xc + c * dial->radius;
      t_y = yc - s * dial->radius;
-     snprintf (buf, sizeof(buf), "%3.0f%%", 
-	       (dial->percentage * 100.0));
+     g_snprintf (buf, sizeof(buf), "%3.0f%%", 
+	         (dial->percentage * 100.0));
 
      i = gdk_string_width (widget->style->font, buf) / 2;
      gdk_draw_string (dial->offscreen_pixmap,
