@@ -4,10 +4,13 @@
 #include <stdio.h>
 #include <limits.h>
 #include <ctype.h>
+#include <errno.h>
 #include <pwd.h>
+#include <signal.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/file.h>
+#include <sys/resource.h>
 #include <fcntl.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
@@ -16,6 +19,8 @@
 CORBA_ORB gnome_orbit_orb;
 CORBA_Principal request_cookie;
 
+extern void goad_register_arguments(void);
+
 static void
 orb_handle_connection(GIOPConnection *cnx, gint source, GdkInputCondition cond)
 {
@@ -35,16 +40,16 @@ orb_handle_connection(GIOPConnection *cnx, gint source, GdkInputCondition cond)
 
 static void orb_add_connection(GIOPConnection *cnx)
 {
-  cnx->user_data = (gpointer)gtk_input_add_full(GIOP_CONNECTION_GET_FD(cnx),
+  cnx->user_data = GINT_TO_POINTER (gtk_input_add_full(GIOP_CONNECTION_GET_FD(cnx),
 						GDK_INPUT_READ|GDK_INPUT_EXCEPTION,
 						(GdkInputFunction)orb_handle_connection,
-						NULL, cnx, NULL);
+						NULL, cnx, NULL));
 }
 
 static void orb_remove_connection(GIOPConnection *cnx)
 {
-  gtk_input_remove((guint)cnx->user_data);
-  cnx->user_data = (gpointer)-1;
+  gtk_input_remove(GPOINTER_TO_UINT (cnx->user_data));
+  cnx->user_data = GINT_TO_POINTER (-1);
 }
 
 static CORBA_boolean
@@ -66,24 +71,25 @@ gnome_ORBit_request_validate(CORBA_unsigned_long request_id,
 static void
 get_exclusive_lock (int fd)
 {
-  /* flock (fd, LOCK_EX); */
-  struct flock lbuf;
-  lbuf.l_type = F_WRLCK;
-  lbuf.l_whence = SEEK_SET;
-  lbuf.l_start = lbuf.l_len = 0L; /* Lock the whole file.  */
-  fcntl (fd, F_SETLKW, &lbuf);
+	/* flock (fd, LOCK_EX); */
+	struct flock lbuf;
 
+	lbuf.l_type = F_WRLCK;
+	lbuf.l_whence = SEEK_SET;
+	lbuf.l_start = lbuf.l_len = 0L; /* Lock the whole file.  */
+	fcntl (fd, F_SETLKW, &lbuf);
 }
 
 static void
 release_lock (int fd)
 {
-  /* flock (fd, LOCK_UN); */
-  struct flock lbuf;
-  lbuf.l_type = F_UNLCK;
-  lbuf.l_whence = SEEK_SET;
-  lbuf.l_start = lbuf.l_len = 0L; /* Unlock the whole file.  */
-  fcntl (fd, F_SETLKW, &lbuf);
+	/* flock (fd, LOCK_UN); */
+	struct flock lbuf;
+
+	lbuf.l_type = F_UNLCK;
+	lbuf.l_whence = SEEK_SET;
+	lbuf.l_start = lbuf.l_len = 0L; /* Unlock the whole file.  */
+	fcntl (fd, F_SETLKW, &lbuf);
 }
 
 /*
@@ -110,6 +116,7 @@ get_cookie_reliably (void)
 	fd = open (name, O_CREAT|O_EXCL|O_WRONLY, S_IRUSR | S_IWUSR );
 	if (fd >= 0){
 		unsigned int i;
+		int v;
 
 		get_exclusive_lock (fd);
 		srandom (time (NULL));
@@ -118,16 +125,39 @@ get_cookie_reliably (void)
 		
 		random_string [sizeof(random_string)-1] = 0;
 
-		write (fd, random_string, sizeof (random_string)-1);
+		while (1){
+			v = write (fd, random_string, sizeof (random_string)-1);
+			if (v == sizeof (random_string)-1)
+				break;
+			if (v == 0)
+				g_error (_("Can not write to the cookie file"));
+			if (v == -1 && errno != EINTR){
+				perror (_("Unknown error while writing cookie file"));
+				exit (1);
+			}
+		}
 		close (fd);
 		release_lock (fd);
 	} else {
+		int v;
+		
 		fd = open (name, O_RDONLY);
 		if (fd == -1)
-			g_error ("Could not open the cookie file");
+			g_error (_("Could not open the cookie file"));
 		get_exclusive_lock (fd);
+
 		memset (random_string, 0, sizeof (random_string));
-		read (fd, random_string, sizeof (random_string)-1);
+
+		while (1){
+			v = read (fd, random_string, sizeof (random_string)-1);
+			if (v == 0)
+				g_error (_("Can not read the cookie file\n"));
+			if (v == -1 && errno != EINTR){
+				perror (_("While reading the cookie file\n"));
+				exit (1);
+			}
+			break;
+		}
 		release_lock (fd);
 		close (fd);
 	}
@@ -135,7 +165,7 @@ get_cookie_reliably (void)
 	return g_strdup (random_string);
 }
 
-/*** gnome_CORBA_init
+/*** do_CORBA_init
      Description: Sets up the ORBit connection add/remove function pointers
                   to our routines, which inform the gtk main loop about
 		  the CORBA connection fd's.
@@ -144,74 +174,75 @@ get_cookie_reliably (void)
 
 		  Sets up a cookie for requests.
  */
-CORBA_ORB
-gnome_CORBA_init(char *app_id,
-		 struct argp *app_parser,
-		 int *argc, char **argv,
-		 unsigned int flags,
-		 int *arg_index,
-		 CORBA_Environment *ev)
+static CORBA_ORB
+do_CORBA_init(int *argc, char **argv,
+	      GnorbaInitFlags flags,
+	      CORBA_Environment *ev)
 {
   CORBA_ORB retval;
-  
+
   IIOPAddConnectionHandler = orb_add_connection;
   IIOPRemoveConnectionHandler = orb_remove_connection;
 
-  gnome_init(app_id, app_parser, *argc, argv, flags, arg_index);
-
   gnome_orbit_orb = retval = CORBA_ORB_init(argc, argv, "orbit-local-orb", ev);
 
-  request_cookie._buffer = get_cookie_reliably ();
-
-  g_assert(request_cookie._buffer && *request_cookie._buffer);
-
-  request_cookie._length = strlen(request_cookie._buffer) + 1;
-
-  ORBit_set_request_validation_handler(&gnome_ORBit_request_validate);
-  ORBit_set_default_principal(&request_cookie);
-
-  return retval;
-}
-
-/* This routine bootstraps CORBA connectivity for a GNOME desktop session... */
-CORBA_Object
-gnome_name_service_get(void)
-{
-  static CORBA_Object name_service = CORBA_OBJECT_NIL;
-  CORBA_Object retval = NULL;
-  char *ior;
-  GdkAtom propname, proptype;
-  CORBA_Environment ev;
-
-  g_return_val_if_fail(gnome_orbit_orb, CORBA_OBJECT_NIL);
-
-  CORBA_exception_init(&ev);
-
-  propname = gdk_atom_intern("CORBA_NAME_SERVICE", FALSE);
-  proptype = gdk_atom_intern("STRING", FALSE);
-
-  if(CORBA_Object_is_nil(name_service, &ev)) {
-
-    /* First, try and see if another application has started the name service
-       (and indicated its presence by setting a root window property */
-    gdk_property_get(GDK_ROOT_PARENT(), propname, proptype,
-		     0, 9999, FALSE, NULL,
-		     NULL, NULL, (guchar **)&ior);
-    if(!ior)
-      goto out;
-
-    name_service = CORBA_ORB_string_to_object(gnome_orbit_orb, ior, &ev);
-
-    if(ev._major != CORBA_NO_EXCEPTION)
-      name_service = CORBA_OBJECT_NIL;
+  if(!(flags & GNORBA_INIT_DISABLE_COOKIES)) {
+    request_cookie._buffer = get_cookie_reliably ();
+    
+    g_assert(request_cookie._buffer && *request_cookie._buffer);
+    
+    request_cookie._length = strlen(request_cookie._buffer) + 1;
+    
+    ORBit_set_request_validation_handler(&gnome_ORBit_request_validate);
+    ORBit_set_default_principal(&request_cookie);
   }
 
- out:
-  if(!CORBA_Object_is_nil(name_service, &ev))
-    retval = CORBA_Object_duplicate(name_service, &ev);
-  else
-    g_error("Could not get name service!");
-
-  CORBA_exception_free(&ev);
   return retval;
 }
+
+CORBA_ORB
+gnome_CORBA_init(const char *app_id,
+		 const char *app_version,
+		 int *argc, char **argv,
+		 GnorbaInitFlags gnorba_flags,
+		 CORBA_Environment *ev)
+{
+  CORBA_ORB retval;
+
+  if(gnorba_flags & GNORBA_INIT_SERVER_FUNC)
+    goad_register_arguments();
+
+  gnome_init(app_id, app_version, *argc, argv);
+  retval = do_CORBA_init(argc, argv, gnorba_flags, ev);
+
+  return retval;
+}
+
+CORBA_ORB
+gnome_CORBA_init_with_popt_table(const char *app_id,
+				 const char *app_version,
+				 int *argc, char **argv,
+				 const struct poptOption *options,
+				 int popt_flags,
+				 poptContext *return_ctx,
+				 GnorbaInitFlags gnorba_flags,
+				 CORBA_Environment *ev)
+{
+  CORBA_ORB retval;
+
+  if(gnorba_flags & GNORBA_INIT_SERVER_FUNC)
+    goad_register_arguments();
+
+  gnome_init_with_popt_table(app_id, app_version, *argc, argv, options,
+			     popt_flags, return_ctx);
+  retval = do_CORBA_init(argc, argv, gnorba_flags, ev);
+
+  return retval;
+}
+
+CORBA_ORB
+gnome_CORBA_ORB(void)
+{
+  return gnome_orbit_orb;
+}
+
