@@ -16,12 +16,14 @@ static void
 test_win (GnomeCanvasItem **board)
 {
 	int i;
+	GtkWidget *dlg;
 
 	for (i = 0; i < 15; i++)
 		if (!board[i] || (GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (board[i]), "piece_num")) != i))
 			return;
-
-	gnome_dialog_run_modal (GNOME_DIALOG (gnome_ok_dialog ("You stud, you win!")));
+	dlg=gnome_ok_dialog ("You stud, you win!");
+	gtk_window_set_modal(GTK_WINDOW(dlg),TRUE);
+	gnome_dialog_run (GNOME_DIALOG (dlg));
 }
 
 static char *
@@ -182,7 +184,7 @@ create_canvas_fifteen (void)
 	char buf[20];
 
 	vbox = gtk_vbox_new (FALSE, 4);
-	gtk_container_border_width (GTK_CONTAINER (vbox), 4);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
 	gtk_widget_show (vbox);
 
 	alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
@@ -197,7 +199,7 @@ create_canvas_fifteen (void)
 	/* Create the canvas and board */
 
 	canvas = gnome_canvas_new ();
-	gnome_canvas_set_size (GNOME_CANVAS (canvas), PIECE_SIZE * 4 + 1, PIECE_SIZE * 4 + 1);
+	gtk_widget_set_usize (canvas, PIECE_SIZE * 4 + 1, PIECE_SIZE * 4 + 1);
 	gnome_canvas_set_scroll_region (GNOME_CANVAS (canvas), 0, 0, PIECE_SIZE * 4 + 1, PIECE_SIZE * 4 + 1);
 	gtk_container_add (GTK_CONTAINER (frame), canvas);
 	gtk_widget_show (canvas);
