@@ -183,9 +183,16 @@ setup_divisions (GnomeCanvasGroup *root)
 	setup_heading (group, "Widgets", 8);
 }
 
+#define gray50_width 2
+#define gray50_height 2
+static char gray50_bits[] = {
+  0x02, 0x01, };
+
 static void
 setup_rectangles (GnomeCanvasGroup *root)
 {
+	GdkBitmap *stipple;
+
 	setup_item (gnome_canvas_item_new (root,
 					   gnome_canvas_rect_get_type (),
 					   "x1", 20.0,
@@ -196,6 +203,7 @@ setup_rectangles (GnomeCanvasGroup *root)
 					   "width_pixels", 8,
 					   NULL));
 
+	stipple = gdk_bitmap_create_from_data (NULL, gray50_bits, gray50_width, gray50_height);
 	setup_item (gnome_canvas_item_new (root,
 					   gnome_canvas_rect_get_type (),
 					   "x1", 90.0,
@@ -203,9 +211,11 @@ setup_rectangles (GnomeCanvasGroup *root)
 					   "x2", 180.0,
 					   "y2", 100.0,
 					   "fill_color", "mediumseagreen",
+					   "fill_stipple", stipple,
 					   "outline_color", "black",
 					   "width_units", 4.0,
 					   NULL));
+	gdk_bitmap_unref (stipple);
 
 	setup_item (gnome_canvas_item_new (root,
 					   gnome_canvas_rect_get_type (),
@@ -220,6 +230,8 @@ setup_rectangles (GnomeCanvasGroup *root)
 static void
 setup_ellipses (GnomeCanvasGroup *root)
 {
+	GdkBitmap *stipple;
+
 	setup_item (gnome_canvas_item_new (root,
 					   gnome_canvas_ellipse_get_type (),
 					   "x1", 220.0,
@@ -241,6 +253,7 @@ setup_ellipses (GnomeCanvasGroup *root)
 					   "width_units", 4.0,
 					   NULL));
 
+	stipple = gdk_bitmap_create_from_data (NULL, gray50_bits, gray50_width, gray50_height);
 	setup_item (gnome_canvas_item_new (root,
 					   gnome_canvas_ellipse_get_type (),
 					   "x1", 210.0,
@@ -248,7 +261,11 @@ setup_ellipses (GnomeCanvasGroup *root)
 					   "x2", 280.0,
 					   "y2", 140.0,
 					   "fill_color", "cadetblue",
+					   "fill_stipple", stipple,
+					   "outline_color", "black",
+					   "width_pixels", 0,
 					   NULL));
+	gdk_bitmap_unref (stipple);
 }
 
 static GnomeCanvasGroup *
@@ -279,6 +296,9 @@ make_anchor (GnomeCanvasGroup *root, double x, double y)
 static void
 setup_texts (GnomeCanvasGroup *root)
 {
+	GdkBitmap *stipple;
+
+	stipple = gdk_bitmap_create_from_data (NULL, gray50_bits, gray50_width, gray50_height);
 	gnome_canvas_item_new (make_anchor (root, 420.0, 20.0),
 			       gnome_canvas_text_get_type (),
 			       "text", "Anchor NW",
@@ -287,7 +307,9 @@ setup_texts (GnomeCanvasGroup *root)
 			       "font", "-adobe-helvetica-bold-r-normal--24-240-75-75-p-138-iso8859-1",
 			       "anchor", GTK_ANCHOR_NW,
 			       "fill_color", "blue",
+			       "fill_stipple", stipple,
 			       NULL);
+	gdk_bitmap_unref (stipple);
 
 	gnome_canvas_item_new (make_anchor (root, 470.0, 75.0),
 			       gnome_canvas_text_get_type (),
@@ -421,6 +443,7 @@ make_hilbert (GnomeCanvasGroup *root)
 	char *c;
 	double *pp, *p;
 	GnomeCanvasPoints *points;
+	GdkBitmap *stipple;
 
 	points = gnome_canvas_points_new (strlen (hilbert) + 1);
 	points->coords[0] = 340.0;
@@ -450,14 +473,17 @@ make_hilbert (GnomeCanvasGroup *root)
 			break;
 		}
 
+	stipple = gdk_bitmap_create_from_data (NULL, gray50_bits, gray50_width, gray50_height);
 	setup_item (gnome_canvas_item_new (root,
 					   gnome_canvas_line_get_type (),
 					   "points", points,
 					   "fill_color", "red",
-					   "width_units", 3.0,
+					   "fill_stipple", stipple,
+					   "width_units", 4.0,
 					   "cap_style", GDK_CAP_PROJECTING,
 					   "join_style", GDK_JOIN_MITER,
 					   NULL));
+	gdk_bitmap_unref (stipple);
 
 	gnome_canvas_points_free (points);
 }
@@ -529,6 +555,69 @@ setup_lines (GnomeCanvasGroup *root)
 	gnome_canvas_points_free (points);
 }
 
+static void
+setup_polygons (GnomeCanvasGroup *root)
+{
+	GnomeCanvasPoints *points;
+	GdkBitmap *stipple;
+
+	points = gnome_canvas_points_new (3);
+	points->coords[0] = 210.0;
+	points->coords[1] = 320.0;
+	points->coords[2] = 210.0;
+	points->coords[3] = 380.0;
+	points->coords[4] = 260.0;
+	points->coords[5] = 350.0;
+	stipple = gdk_bitmap_create_from_data (NULL, gray50_bits, gray50_width, gray50_height);
+	setup_item (gnome_canvas_item_new (root,
+					   gnome_canvas_polygon_get_type (),
+					   "points", points,
+					   "fill_color", "blue",
+					   "fill_stipple", stipple,
+					   "outline_color", "black",
+					   NULL));
+	gdk_bitmap_unref (stipple);
+	gnome_canvas_points_free (points);
+
+	points = gnome_canvas_points_new (14);
+	points->coords[0] = 270.0;
+	points->coords[1] = 330.0;
+	points->coords[2] = 270.0;
+	points->coords[3] = 430.0;
+	points->coords[4] = 390.0;
+	points->coords[5] = 430.0;
+	points->coords[6] = 390.0;
+	points->coords[7] = 330.0;
+	points->coords[8] = 310.0;
+	points->coords[9] = 330.0;
+	points->coords[10] = 310.0;
+	points->coords[11] = 390.0;
+	points->coords[12] = 350.0;
+	points->coords[13] = 390.0;
+	points->coords[14] = 350.0;
+	points->coords[15] = 370.0;
+	points->coords[16] = 330.0;
+	points->coords[17] = 370.0;
+	points->coords[18] = 330.0;
+	points->coords[19] = 350.0;
+	points->coords[20] = 370.0;
+	points->coords[21] = 350.0;
+	points->coords[22] = 370.0;
+	points->coords[23] = 410.0;
+	points->coords[24] = 290.0;
+	points->coords[25] = 410.0;
+	points->coords[26] = 290.0;
+	points->coords[27] = 330.0;
+	setup_item (gnome_canvas_item_new (root,
+					   gnome_canvas_polygon_get_type (),
+					   "points", points,
+					   "fill_color", "tan",
+					   "outline_color", "black",
+					   "width_units", 3.0,
+					   NULL));
+	gnome_canvas_points_free (points);
+}
+
 static void
 setup_widgets (GnomeCanvasGroup *root)
 {
@@ -568,7 +657,7 @@ key_press (GnomeCanvas *canvas, GdkEventKey *event, gpointer data)
 }
 
 GtkWidget *
-create_canvas_primitives (void)
+create_canvas_primitives (gint aa)
 {
 	GtkWidget *vbox;
 	GtkWidget *hbox;
@@ -580,7 +669,7 @@ create_canvas_primitives (void)
 	GnomeCanvasGroup *root;
 
 	vbox = gtk_vbox_new (FALSE, 4);
-	gtk_container_border_width (GTK_CONTAINER (vbox), 4);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
 	gtk_widget_show (vbox);
 
 	w = gtk_label_new ("Drag an item with button 1.  Click button 2 on an item to lower it,\n"
@@ -593,9 +682,15 @@ create_canvas_primitives (void)
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 	gtk_widget_show (hbox);
 
-	gtk_widget_push_visual (gdk_imlib_get_visual ());
-	gtk_widget_push_colormap (gdk_imlib_get_colormap ());
+	if (aa) {
+		gtk_widget_push_visual (gdk_rgb_get_visual ());
+		gtk_widget_push_colormap (gdk_rgb_get_cmap ());
+	} else {
+		gtk_widget_push_visual (gdk_imlib_get_visual ());
+		gtk_widget_push_colormap (gdk_imlib_get_colormap ());
+	}
 	canvas = gnome_canvas_new ();
+	GNOME_CANVAS (canvas)->aa = aa;
 	gtk_widget_pop_colormap ();
 	gtk_widget_pop_visual ();
 
@@ -631,7 +726,7 @@ create_canvas_primitives (void)
 			  0, 0);
 	gtk_widget_show (frame);
 
-	gnome_canvas_set_size (GNOME_CANVAS (canvas), 600, 450);
+	gtk_widget_set_usize (canvas, 600, 450);
 	gnome_canvas_set_scroll_region (GNOME_CANVAS (canvas), 0, 0, 600, 450);
 	gtk_container_add (GTK_CONTAINER (frame), canvas);
 	gtk_widget_show (canvas);
@@ -666,6 +761,7 @@ create_canvas_primitives (void)
 	setup_texts (root);
 	setup_images (root);
 	setup_lines (root);
+	setup_polygons (root);
 	setup_widgets (root);
 
 	GTK_WIDGET_SET_FLAGS (canvas, GTK_CAN_FOCUS);
