@@ -30,6 +30,7 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/resource.h>
+#include <limits.h>
 #include <unistd.h>
 #include <signal.h>
 #include <fcntl.h>
@@ -222,11 +223,7 @@ open_ptys (int update_db)
 	pty_info *p;
 	int result;
 
-#ifdef _SC_TTY_NAME_MAX
-	term_name = alloca (sysconf (_SC_TTY_NAME_MAX) + 1);
-#else
 	term_name = alloca (PATH_MAX) + 1;
-#endif
 	
 	status = openpty (&master_pty, &slave_pty, term_name, NULL, NULL);
 	if (status == -1){
